
/*

A glossary is a text organised alphabetically into sections, one for each entity

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class Glossary
{ private String textString;
  private String ontologyIRI;
  private Vector<Verb> verbs;
  private Vector<GlossaryEntry> section;
  private OntologyText ontologyText;
  private Lexicon lexicon;

  public Glossary(OntologyText arg1, Lexicon arg2)
  { ontologyText = arg1;
    lexicon = arg2;
    ontologyIRI = ontologyText.getOntologyIRI();
    verbs = ontologyText.getVerbs();
    section = new Vector<GlossaryEntry>();
    makeGlossary();
    textString = makeTextString();
  }

  public String getTextString() {return textString; }

  private void makeGlossary()
  { Vector<String> individualList = lexicon.getIndividualList();
    Vector<String> classList = lexicon.getClassList(OwlEnglish.SINGULAR);
    Vector<String> objectPropertyList = lexicon.getObjectPropertyList(OwlEnglish.SINGULAR);
    Vector<String> dataPropertyList = lexicon.getDataPropertyList(OwlEnglish.SINGULAR);
    GlossaryEntry entry;
    for (int i=0; i<classList.size(); i++)
    { entry = new GlossaryEntry(OwlTerm.CLASS,classList.elementAt(i),ontologyText);
      section.addElement(entry);
    }
    for (int i=0; i<individualList.size(); i++)
    { entry = new GlossaryEntry(OwlTerm.NAMED_INDIVIDUAL,individualList.elementAt(i),ontologyText);
      section.addElement(entry);
    }
    for (int i=0; i<objectPropertyList.size(); i++)
    { entry = new GlossaryEntry(OwlTerm.OBJECT_PROPERTY,objectPropertyList.elementAt(i),ontologyText);
      section.addElement(entry);
    }
    for (int i=0; i<dataPropertyList.size(); i++)
    { entry = new GlossaryEntry(OwlTerm.DATA_PROPERTY,dataPropertyList.elementAt(i),ontologyText);
      section.addElement(entry);
    }
  }

  private String makeTextString()
  { String s = "# OntologyIRI " + ontologyIRI + "\n";
    for (int i=0; i<verbs.size(); i++)
    s = s + verbDeclaration(verbs.elementAt(i));
    for (int i=0; i<section.size(); i++)
    s = s + "\n" + section.elementAt(i).getTextString();
    return s;
  }

  private String verbDeclaration(Verb verb)
  { String s = "# VERB ";
    s = s + verb.getPlural() + " " + verb.getSingular() + "\n";
    return s;
  }

}
