
/*

A glossary entry holds the text relating to a single entity (class, individual or property)

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class GlossaryEntry
{ private String textString;
  private String entityName;
  private String entityOwlSyntax;
  private int entityType;
  private Vector<Sentence> entrySentences;
  private OntologyText ontologyText;

  public GlossaryEntry(int arg1, String arg2, OntologyText arg3)
  { entityType = arg1;
    entityName = arg2;
    ontologyText = arg3;
    entrySentences = new Vector<Sentence>();
    addSentences(OwlTerm.EQUIVALENT_CLASSES);
    addSentences(OwlTerm.SUB_CLASS_OF);
    addSentences(OwlTerm.CLASS_ASSERTION);
    addSentences(OwlTerm.DISJOINT_CLASSES);
    addSentences(OwlTerm.OBJECT_PROPERTY_DOMAIN);
    addSentences(OwlTerm.OBJECT_PROPERTY_RANGE);
    addSentences(OwlTerm.DATA_PROPERTY_DOMAIN);
    addSentences(OwlTerm.DATA_PROPERTY_RANGE);
    addSentences(OwlTerm.FUNCTIONAL_OBJECT_PROPERTY);
    addSentences(OwlTerm.INVERSE_FUNCTIONAL_OBJECT_PROPERTY);
    addSentences(OwlTerm.SYMMETRIC_OBJECT_PROPERTY);
    addSentences(OwlTerm.ASYMMETRIC_OBJECT_PROPERTY);
    addSentences(OwlTerm.REFLEXIVE_OBJECT_PROPERTY);
    addSentences(OwlTerm.IRREFLEXIVE_OBJECT_PROPERTY);
    addSentences(OwlTerm.TRANSITIVE_OBJECT_PROPERTY);
    addSentences(OwlTerm.FUNCTIONAL_DATA_PROPERTY);
    addSentences(OwlTerm.SUB_OBJECT_PROPERTY_OF);
    addSentences(OwlTerm.DISJOINT_OBJECT_PROPERTIES);
    addSentences(OwlTerm.EQUIVALENT_OBJECT_PROPERTIES);
    addSentences(OwlTerm.INVERSE_OBJECT_PROPERTIES);
    textString = makeTextString();
  }

  public String getTextString() {return textString; }
  public String getEntityName() {return entityName; }
  public int getEntityType() {return entityType; }

  private void addSentences(int functor)
  { int numsen = ontologyText.getNumberOfSentences();
    Sentence sent;
    OwlTerm term;
    for (int i=0; i<numsen; i++)
    { sent = ontologyText.getSentence(i);
      if (sent.sentenceIsCorrect() == true)
      { term = sent.getOwlTerm();
        if ((term.getFunctor() == functor) &&
            (entityName.equals(termSubjectName(term))))
        entrySentences.addElement(sent);
      }
    }
  }

  private String termSubjectName(OwlTerm term)
  { String s = "";
    if ((term.getNumberOfArguments() > 0) &&
        (term.getArgument(0).getFunctor() == entityType))
    s = term.getArgument(0).getEntityName();
    return s;
  }

  private String makeTextString()
  { String s = "";
    s = s + makeComment();
    Sentence sent;
    for (int i=0; i<entrySentences.size(); i++)
    { sent = entrySentences.elementAt(i);
      s = s + sent.getGeneratedSentence() + "\n";
    }
    return s;
  }

  private String makeComment()
  { String s = "% ";
    s = s + entityName.toUpperCase() + "  ";
    OwlTerm term = new OwlTerm(entityType,0);
    term.setEntityName(entityName);
    s = s + term.getPrintForm() + "\n";
    return s;
  }
      
}
