
/*
Lexicon constructs vectors of entity names for individuals, classes, properties

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class Lexicon
{ private Vector<String> individualList;
  private Vector<String> classList;
  private Vector<String> objectPropertyList;
  private Vector<String> dataPropertyList;
  private Vector<String> classListPlural;
  private Vector<String> objectPropertyListPlural;
  private Vector<String> dataPropertyListPlural;
  private OntologyText text;

  public Lexicon(OntologyText textArg)
  { text = textArg;
    individualList = new Vector<String>();
    classList = new Vector<String>();
    objectPropertyList = new Vector<String>();
    dataPropertyList = new Vector<String>();
    classListPlural = new Vector<String>();
    objectPropertyListPlural = new Vector<String>();
    dataPropertyListPlural = new Vector<String>();
    for (int i=0; i<text.getNumberOfSentences(); i++)
    updateLists(text.getSentence(i));
  }

  public Vector<String> getIndividualList() {return individualList; }

  public Vector<String> getClassList(int number)
  { if (number == OwlEnglish.SINGULAR) return classList;
    else return classListPlural;
  }

  public Vector<String> getObjectPropertyList(int number)
  { if (number == OwlEnglish.SINGULAR) return objectPropertyList;
    else return objectPropertyListPlural;
  }

  public Vector<String> getDataPropertyList(int number)
  { if (number == OwlEnglish.SINGULAR) return dataPropertyList;
    else return dataPropertyListPlural;
  }

  private void updateLists(Sentence s)
  {if (s.sentenceIsCorrect())
    { OwlTerm term = s.getOwlTerm();
      updateIndividualList(term);
      updateClassList(term);
      updateObjectPropertyList(term);
      updateDataPropertyList(term);
    }
  }

  private void updateIndividualList(OwlTerm term)
  { if ((term.getFunctor() == OwlTerm.NAMED_INDIVIDUAL) && 
       (! term.getEntityName().equals("[Individual]")))
    addToIndividualList(term.getEntityName());
    else if (term.getNumberOfArguments() > 0)
    { for (int i=0; i<term.getNumberOfArguments(); i++)
      updateIndividualList(term.getArgument(i));
    }
    if (term.getFunctor() == OwlTerm.METADATA)
    { String[] split = term.getEntityName().split(" ", 2);
      if (split.length > 1)
      { if (split[0].toLowerCase().equals("namedindividual"))
        addMetaDataToIndividualList(split[1]);
      }
    }
  }

  private void addMetaDataToIndividualList(String individualName)
  { String s = individualName + " is a thing.";
    Sentence sen = new Sentence(0,s,0,text.getVerbs());
    OwlTerm term = sen.getOwlTerm();
    if (term.getFunctor()==OwlTerm.CLASS_ASSERTION)
    { OwlTerm arg = term.getArgument(0);
      addToIndividualList(arg.getEntityName());
    }
  }

// Go through list until you reach either the final element, or one that is equal
// to the current string, or one that is alphabetically later. Then place the current
// string accordingly.
  private void addToIndividualList(String s)
  { if (individualList.isEmpty())
    individualList.addElement(s);
    else
    { int i = 0;
      while ((i < individualList.size()-1) && (s.compareToIgnoreCase(individualList.elementAt(i)) > 0)) i++;
      if (s.compareToIgnoreCase(individualList.elementAt(i)) < 0)
      individualList.add(i,s);
      else if (s.compareToIgnoreCase(individualList.elementAt(i)) > 0)
      individualList.addElement(s);
    }
  }

  private void updateClassList(OwlTerm term)
  { if ((term.getFunctor() == OwlTerm.CLASS) && 
       (! term.getEntityName().equals("[class]")))
    addToClassList(term.getEntityName(),term.getPluralEntityName());
    else if (term.getNumberOfArguments() > 0)
    { for (int i=0; i<term.getNumberOfArguments(); i++)
      updateClassList(term.getArgument(i));
    };
    if (term.getFunctor() == OwlTerm.METADATA)
    { String[] split = term.getEntityName().split(" ", 2);
      if (split.length > 1)
      { if (split[0].toLowerCase().equals("class"))
        addMetaDataToClassList(split[1]);
      }
    }
  }

  private void addMetaDataToClassList(String className)
  { String s = "A " + className + " is a thing.";
    Sentence sen = new Sentence(0,s,0,text.getVerbs());
    OwlTerm term = sen.getOwlTerm();
    if (term.getFunctor()==OwlTerm.SUB_CLASS_OF)
    { OwlTerm arg = term.getArgument(0);
      addToClassList(arg.getEntityName(),arg.getPluralEntityName());
    }
  }    

  private void addToClassList(String s, String p)
  { if (classList.isEmpty())
    { classList.addElement(s);
      classListPlural.addElement(p);
    }
    else
    { int i = 0;
      while ((i < classList.size()-1) && (s.compareToIgnoreCase(classList.elementAt(i)) > 0)) i++;
      if (s.compareToIgnoreCase(classList.elementAt(i)) < 0)
      { classList.add(i,s);
        classListPlural.add(i,p);
      }
      else if (s.compareToIgnoreCase(classList.elementAt(i)) > 0)
      { classList.addElement(s);
        classListPlural.addElement(p);
      }
    }
  }

  private void updateObjectPropertyList(OwlTerm term)
  { if ((term.getFunctor() == OwlTerm.OBJECT_PROPERTY) &&
       (! term.getEntityName().equals("[has-property]")))
    addToObjectPropertyList(term.getEntityName(),term.getPluralEntityName());
    else if (term.getNumberOfArguments() > 0)
    { for (int i=0; i<term.getNumberOfArguments(); i++)
      updateObjectPropertyList(term.getArgument(i));
    };
    if (term.getFunctor() == OwlTerm.METADATA)
    { String[] split = term.getEntityName().split(" ", 2);
      if (split.length > 1)
      { if (split[0].toLowerCase().equals("objectproperty"))
        addMetaDataToObjectPropertyList(split[1]);
      }
    }
  }

  private void addMetaDataToObjectPropertyList(String propertyName)
  { String s = "The property <" + propertyName + "> is functional.";
    Sentence sen = new Sentence(0,s,0,text.getVerbs());
    OwlTerm term = sen.getOwlTerm();
    if (term.getFunctor()==OwlTerm.FUNCTIONAL_OBJECT_PROPERTY)
    { OwlTerm arg = term.getArgument(0);
      addToObjectPropertyList(arg.getEntityName(),arg.getPluralEntityName());
    }
  }    

  private void addToObjectPropertyList(String s, String p)
  { if (objectPropertyList.isEmpty())
    { objectPropertyList.addElement(s);
      objectPropertyListPlural.addElement(p);
    }
    else
    { int i = 0;
      while ((i < objectPropertyList.size()-1) && (s.compareToIgnoreCase(objectPropertyList.elementAt(i)) > 0)) i++;
      if (s.compareToIgnoreCase(objectPropertyList.elementAt(i)) < 0)
      { objectPropertyList.add(i,s);
        objectPropertyListPlural.add(i,p);
      }
      else if (s.compareToIgnoreCase(objectPropertyList.elementAt(i)) > 0)
      { objectPropertyList.addElement(s);
        objectPropertyListPlural.addElement(p);
      }
    }
  }

  private void updateDataPropertyList(OwlTerm term)
  { if ((term.getFunctor() == OwlTerm.DATA_PROPERTY) &&
       (! term.getEntityName().equals("[has-data-property]")))
    addToDataPropertyList(term.getEntityName(),term.getPluralEntityName());
    else if (term.getNumberOfArguments() > 0)
    { for (int i=0; i<term.getNumberOfArguments(); i++)
      updateDataPropertyList(term.getArgument(i));
    };
    if (term.getFunctor() == OwlTerm.METADATA)
    { String[] split = term.getEntityName().split(" ", 2);
      if (split.length > 1)
      { if (split[0].toLowerCase().equals("dataproperty"))
        addMetaDataToDataPropertyList(split[1]);
      }
    }
  }

  private void addMetaDataToDataPropertyList(String propertyName)
  { String s = "The data-property <" + propertyName + "> is functional.";
    Sentence sen = new Sentence(0,s,0,text.getVerbs());
    OwlTerm term = sen.getOwlTerm();
    if (term.getFunctor()==OwlTerm.FUNCTIONAL_DATA_PROPERTY)
    { OwlTerm arg = term.getArgument(0);
      addToDataPropertyList(arg.getEntityName(),arg.getPluralEntityName());
    }
  }    

  private void addToDataPropertyList(String s, String p)
  { if (dataPropertyList.isEmpty())
    { dataPropertyList.addElement(s);
      dataPropertyListPlural.addElement(p);
    }
    else
    { int i = 0;
      while ((i < dataPropertyList.size()-1) && (s.compareToIgnoreCase(dataPropertyList.elementAt(i)) > 0)) i++;
      if (s.compareToIgnoreCase(dataPropertyList.elementAt(i)) < 0)
      { dataPropertyList.add(i,s);
        dataPropertyListPlural.add(i,p);
      }
      else if (s.compareToIgnoreCase(dataPropertyList.elementAt(i)) > 0)
      { dataPropertyList.addElement(s);
        dataPropertyListPlural.addElement(p);
      }
    }
  }

  public void display()
  { for (int i=0; i<classList.size(); i++)
    System.out.println(classList.elementAt(i));
  }

}
