 
/*
User interface for Ontology Editing program
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.swing.text.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class OntologyEditor extends JFrame
{ private Action openAction = new OpenAction();
  private Action saveAction = new SaveAction();
  private Action openXMLAction = new OpenXMLAction();
  private Action saveXMLAction = new SaveXMLAction();
  private JMenuItem viewChecked;
  private JMenuItem viewGrouped;
  private JMenuItem editUndo;
  private JTextPane textComp;
  private JScrollPane scrollingText;
  private JTextComponent messageArea;
  private JScrollPane scrollingList;
  private JList optionJList;
  private DefaultListModel model;
  private StyledDocument doc;
  private MutableAttributeSet standardColour;
  private MutableAttributeSet rejectColour;
  private MutableAttributeSet classColour;
  private MutableAttributeSet propertyColour;
  private MutableAttributeSet individualColour;
  private MutableAttributeSet literalColour;
  private int lastSentenceNumber;
  private boolean settingCaret;
  private boolean initialPlaceHolder;
  public static final int TEXT_FONT = 16;
  private Word currentWord;
  private Sentence currentSentence;
  private OntologyText ontologyText;
  private Lexicon lexicon;
  private String currentTextData;
  private String previousTextData;
//  public static final String DEFAULT_PATH = "C:/Documents and Settings/rp3242/Desktop/java/ontology/";
  public static final String DEFAULT_PATH = "/Users/rp3242/Desktop/Java/ontology/";

  public static void main(String[] args)
  { OntologyEditor editor = new OntologyEditor();
    editor.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    editor.setVisible(true);
  }

  public OntologyEditor()
  { super("Ontology Editor");
    messageArea = createTextComponent("Message", 3, 60);
    textComp = new JTextPane();
    textComp.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createEtchedBorder(),
      BorderFactory.createEmptyBorder(5,5,5,5)));
    doc = textComp.getStyledDocument();

    standardColour = new SimpleAttributeSet();
    StyleConstants.setForeground(standardColour, Color.black);
    StyleConstants.setAlignment(standardColour, StyleConstants.ALIGN_LEFT);
    StyleConstants.setFontSize(standardColour, TEXT_FONT);
    StyleConstants.setFontFamily(standardColour, "sans-serif");
    StyleConstants.setLineSpacing(standardColour, 0.01f);
    StyleConstants.setSpaceBelow(standardColour, 5);
    doc.setParagraphAttributes(0, 0, standardColour, true);

    rejectColour = new SimpleAttributeSet();
    StyleConstants.setForeground(rejectColour, Color.red);
    StyleConstants.setFontSize(rejectColour, TEXT_FONT);
    StyleConstants.setFontFamily(rejectColour, "sans-serif");
//    StyleConstants.setBold(rejectColour, true);

    classColour = new SimpleAttributeSet();
    StyleConstants.setForeground(classColour, new Color(205,133,0));
    StyleConstants.setFontSize(classColour, TEXT_FONT);
    StyleConstants.setFontFamily(classColour, "sans-serif");

    propertyColour = new SimpleAttributeSet();
    StyleConstants.setForeground(propertyColour, Color.blue);
    StyleConstants.setFontSize(propertyColour, TEXT_FONT);
    StyleConstants.setFontFamily(propertyColour, "sans-serif");

    individualColour = new SimpleAttributeSet();
    StyleConstants.setForeground(individualColour, Color.magenta);
    StyleConstants.setFontSize(individualColour, TEXT_FONT);
    StyleConstants.setFontFamily(individualColour, "sans-serif");

    literalColour = new SimpleAttributeSet();
    StyleConstants.setForeground(literalColour, new Color(0,170,110));
    StyleConstants.setFontSize(literalColour, TEXT_FONT);
    StyleConstants.setFontFamily(literalColour, "sans-serif");

    scrollingText = new JScrollPane(textComp);

    model = new DefaultListModel();
    model.clear();
    optionJList = new JList(model);
    optionJList.setFont(new Font("sans-serif", Font.PLAIN, TEXT_FONT));
    scrollingList = new JScrollPane(optionJList);
    scrollingList.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createEtchedBorder(),
      BorderFactory.createEmptyBorder(5,5,5,5)));
    int swidth = TEXT_FONT * 45;
    scrollingList.setPreferredSize(new Dimension(swidth, 200));

    addActionNames();

    messageArea.setEditable(false);

    Container content = getContentPane();
    content.add(scrollingText, BorderLayout.CENTER);
    content.add(messageArea, BorderLayout.SOUTH);
    content.add(scrollingList, BorderLayout.EAST);
    setJMenuBar(createMenuBar());

    settingCaret = false;
    initialPlaceHolder = false;
    currentTextData = "";
    previousTextData = "";
    lastSentenceNumber = 0;

    CaretListener editListener = new CaretListener()
    { public void caretUpdate(CaretEvent caretEvent)
// Cut short a recursive call if a caret event is generated within the update procedure
      {if (settingCaret == false)
        { String textData = textComp.getText();
          int caretMark = caretEvent.getMark();
          int caretDot = caretEvent.getDot();

// Recompute the interpretation of the entire text after each caret event
          ontologyText = new OntologyText(textData,caretMark);
          currentSentence = ontologyText.getCurrentSentence();
          lexicon = new Lexicon(ontologyText);

// Note whether the cursor has switched to a new sentence (i.e., new line of text)
// This will determine whether we recolour the whole text
          int currentSentenceNumber = currentSentence.getSentenceNumber();
          boolean newLine;
          if (currentSentenceNumber == lastSentenceNumber) newLine = false;
          else newLine = true;

// Prepare the text to be displayed in the message pane at the bottom
          String owlSyntax = currentSentence.getOwlFunctionalSyntax();
          int cstate = currentSentence.getCurrentState();
          String continuations = ontologyText.getContinuations();
//          String message = cstate + ": " + continuations + "\n\n" + owlSyntax;
          String message = continuations + "\n" + owlSyntax;

// Prepare the model that determines the text displayed in the options pane
          currentWord = ontologyText.getCurrentWord();
          String cws = currentWord.getWordString();
          int cwn = currentWord.getWordNumber();
          if (cws.equals("[class]"))
          addLexiconToModel(lexicon.getClassList(cwn));
          else if (cws.equals("[Individual]"))
          addLexiconToModel(lexicon.getIndividualList());
          else if (cws.equals("[has-property]"))
          addLexiconToModel(lexicon.getObjectPropertyList(cwn));
          else if (cws.equals("[has-data-property]"))
          addLexiconToModel(lexicon.getDataPropertyList(cwn));
          else if (cws.equals("[integer]"))
          addOptionsToModel(Word.integerWord);
          else if (cws.equals("[attribute]"))
          addOptionsToModel(Word.attributeWord);
          else if (cws.equals("[literal]"))
          model.clear();
          else addOptionsToModel(ontologyText.getCurrentSentence().getOptions());

// Recolour current sentence only when more than one word long (why?)
          Word[] words = currentSentence.getWords();
          if (words.length > 1) 
          { SwingUtilities.invokeLater(new Runnable()
            { public void run() { recolourSentence(); } });
          }

// If the cursor is within a place-holder, select the whole place-holder
// This will generate another caret event so we cut off the recursive call by
// flagging settingCaret (see test at start of procedure)
          int start = currentWord.getWordStart();
          int end = currentWord.getWordFinish();
          if (cws.equals("[Individual]") ||
              cws.equals("[class]") ||
              cws.equals("[literal]") ||
              cws.equals("[attribute]") ||
              cws.equals("[integer]") ||
              cws.equals("[has-property]") ||
              cws.equals("[has-data-property]"))
          if ((caretMark>start) && (caretMark<end))
          { settingCaret = true;
            textComp.select(start,end);
            if (start == currentSentence.getStartPosition()) initialPlaceHolder = true;
            else initialPlaceHolder = false; // Controls whether to capitalise replacement
          }

// Write the text constructed above to the message pane
          messageArea.setText(message);

// Recolour the whole text if the caret moves to a new line
          if (newLine == true)
          SwingUtilities.invokeLater(new Runnable() {public void run() {recolourText();}});

// Update the global flags for sentence number and setting caret before ending
          lastSentenceNumber = currentSentenceNumber;
          previousTextData = currentTextData;
          currentTextData = textData;
          settingCaret = false;
        }
      }
    };

    ListSelectionListener optionListener = new ListSelectionListener()
    { public void valueChanged(ListSelectionEvent event)
      { int index = optionJList.getSelectedIndex();
        if (index >= 0)
        { String option = (String) model.get(index);
          if (initialPlaceHolder == true) option = openingCapital(option);
          initialPlaceHolder = false;
          textComp.replaceSelection(option);
          String textData = textComp.getText();
          int caretMark = textComp.getCaretPosition();
          OntologyText ontologyText = new OntologyText(textData,caretMark);
          currentSentence = ontologyText.getCurrentSentence();
          textComp.setCaretPosition(currentSentence.getEndPosition());
        }
      }
    };

    textComp.addCaretListener(editListener);
    optionJList.addListSelectionListener(optionListener);
    setSize(1450, 900);
  }

  private void recolourText()
  { String textData = textComp.getText();
    int caretMark = textComp.getCaretPosition();
    OntologyText ontologyText = new OntologyText(textData,caretMark);
    int nsent = ontologyText.getNumberOfSentences();
    for (int i=0; i<nsent; i++)
    { currentSentence = ontologyText.getSentence(i);
      recolourSentence();
    }
  }

  private void recolourSentence()
  { Word[] words = currentSentence.getWords();
    int slen = words.length;
    if (slen > 1)
//    for (int i=0; i<slen-1; i++)
    for (int i=0; i<slen; i++)
    { if (words[i].getWordRole() == Sentence.ERROR)
      recolourWord(words[i],rejectColour);
      else if (words[i].getWordRole() == Sentence.INDIVIDUAL)
      recolourWord(words[i],individualColour);
      else if (words[i].getWordRole() == Sentence.LITERAL)
      recolourWord(words[i],literalColour);
      else if (words[i].getWordRole() == Sentence.CLASS)
      recolourWord(words[i],classColour);
      else if (words[i].getWordRole() == Sentence.PROPERTY)
      recolourWord(words[i],propertyColour);
      else
      recolourWord(words[i],standardColour);
    }
   recolourWord(currentWord,standardColour);
  }

  private void recolourWord(Word word, AttributeSet aset)
  { int wordStart = word.getWordStart();
    int wordLength = word.getWordFinish()-wordStart;
    doc.setCharacterAttributes(wordStart,wordLength,aset,false);
  }

  private void addOptionsToModel(String[] s)
  { model.clear();
    if (! s[0].equals("No options"))
    for (int i=0; i<s.length; i++) model.addElement(s[i]);
  }

  private void addLexiconToModel(Vector<String> v)
  { model.clear();
    for (int i=0; i<v.size(); i++) model.addElement(v.elementAt(i));
  }

// Make sure the opening letter is upper-case at start of sentence
  private String openingCapital(String s)
  { String cs = s.substring(0,1).toUpperCase()+ s.substring(1,s.length());
    return cs;
  }

  protected JTextComponent createTextComponent(String s, int x, int y)
  { JTextArea ta = new JTextArea(s, x, y);
    ta.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createEtchedBorder(),
      BorderFactory.createEmptyBorder(5,5,5,5)));
    ta.setFont(new Font("sans-serif", Font.PLAIN,  TEXT_FONT));
    ta.setLineWrap(true);
    return ta;
  }

  protected void addActionNames()
  { Action a;
    a = textComp.getActionMap().get(DefaultEditorKit.cutAction);
    a.putValue(Action.NAME, "Cut");

    a = textComp.getActionMap().get(DefaultEditorKit.copyAction);
    a.putValue(Action.NAME, "Copy");

    a = textComp.getActionMap().get(DefaultEditorKit.pasteAction);
    a.putValue(Action.NAME, "Paste");

    a = textComp.getActionMap().get(DefaultEditorKit.selectAllAction);
    a.putValue(Action.NAME, "Select All");
  }

  protected JMenuBar createMenuBar()
  { JMenuBar menubar = new JMenuBar();
    JMenu file = new JMenu("File");
    JMenu edit = new JMenu("Edit");
    JMenu view = new JMenu("Generate");
    menubar.add(file);
    menubar.add(edit);
    menubar.add(view);

    viewChecked = new JMenuItem("Corrected text");
    viewGrouped = new JMenuItem("Glossary");
    editUndo = new JMenuItem("Undo");

    viewChecked.addActionListener(new ViewMenuListener());
    viewGrouped.addActionListener(new ViewMenuListener());
    editUndo.addActionListener(new ViewMenuListener());

    file.add(getOpenAction());
    file.add(getSaveAction());
    file.add(getOpenXMLAction());
    file.add(getSaveXMLAction());
    file.add(new ExitAction());
    edit.add(textComp.getActionMap().get(DefaultEditorKit.cutAction));
    edit.add(textComp.getActionMap().get(DefaultEditorKit.copyAction));
    edit.add(textComp.getActionMap().get(DefaultEditorKit.pasteAction));
    edit.add(textComp.getActionMap().get(DefaultEditorKit.selectAllAction));
    edit.add(editUndo);

    view.add(viewChecked);
    view.add(viewGrouped);
    

    return menubar;
  }

 // Subclass can override to use a different open action.
  protected Action getOpenAction() { return openAction; }

 // Subclass can override to use a different save action.
  protected Action getSaveAction() { return saveAction; }

 // Subclass can override to use a different save action.
  protected Action getOpenXMLAction() { return openXMLAction; }

 // Subclass can override to use a different save action.
  protected Action getSaveXMLAction() { return saveXMLAction; }

 // ********** ACTION INNER CLASSES ********** //

 // A very simple exit action
  public class ExitAction extends AbstractAction
  { public ExitAction() { super("Exit"); }
    public void actionPerformed(ActionEvent ev) { System.exit(0); }
  }

 // An action that opens an existing file
  class OpenAction extends AbstractAction
  { public OpenAction() { super("Open Text"); }

 // Query user for a filename and attempt to open and read the file into the text component.
    public void actionPerformed(ActionEvent ev)
    { JFileChooser chooser = new JFileChooser(DEFAULT_PATH);
      if (chooser.showOpenDialog(OntologyEditor.this) != JFileChooser.APPROVE_OPTION) return;
      File file = chooser.getSelectedFile();
      System.out.println(file.getAbsolutePath());
      if (file == null) return;

      try
      { String tmp;
        String textData = null;
        BufferedReader in = new BufferedReader(new FileReader(file));
        tmp = in.readLine();
        textData = tmp;
        while (tmp != null)
        { tmp = in.readLine();
          textData = textData + "\n" + tmp;
        }
        textData = textData.substring(0,textData.length()-5);
        textComp.setText(textData);
        textComp.setCaretPosition(0);
        in.close();
        SwingUtilities.invokeLater(new Runnable() {public void run() {recolourText();}});
      }

      catch (IOException ex)
      { JOptionPane.showMessageDialog(OntologyEditor.this,
          "File Not Found", "ERROR", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

 // An action that opens an existing XML file and converts to Owl English
  class OpenXMLAction extends AbstractAction
  { public OpenXMLAction() { super("Import XML"); }

 // Query user for a filename and attempt to open and read the file into the text component.
    public void actionPerformed(ActionEvent ev)
    { JFileChooser chooser = new JFileChooser(DEFAULT_PATH);
      if (chooser.showOpenDialog(OntologyEditor.this) != JFileChooser.APPROVE_OPTION) return;
      File file = chooser.getSelectedFile();
      if (file == null) return;

      OwlImport imp = new OwlImport(file);
      String textData = imp.getTextString();
// Interpret and regenerate to obtain correct plural forms
      OntologyText ot = new OntologyText(textData,0);
      textData = ot.getGeneratedTextString();
      String hold = currentTextData;
      textComp.setText(textData);
      textComp.setCaretPosition(0);
      SwingUtilities.invokeLater(new Runnable() {public void run() {recolourText();}});
      previousTextData = hold;
    }
  }

// An action that saves the document to a file
  class SaveAction extends AbstractAction
  { public SaveAction() { super("Save Text"); }

// Query user for a filename and attempt to open and write the text
// component's content to the file.
    public void actionPerformed(ActionEvent ev)
    { JFileChooser chooser = new JFileChooser(DEFAULT_PATH);
      if (chooser.showSaveDialog(OntologyEditor.this) != JFileChooser.APPROVE_OPTION) return;
      File file = chooser.getSelectedFile();
      if (file == null) return;

      FileWriter writer = null;
      try
      { writer = new FileWriter(file);
        textComp.write(writer);
      }
      catch (IOException ex)
      { JOptionPane.showMessageDialog(OntologyEditor.this,
          "File Not Saved", "ERROR", JOptionPane.ERROR_MESSAGE);
      }
      finally
      { if (writer != null)
        { try { writer.close(); }
          catch (IOException x) { }
        }
      }
    }
  }


// An action that saves the document to a file
  class SaveXMLAction extends AbstractAction
  { public SaveXMLAction() { super("Save XML"); }

// Query user for a filename and attempt to open and write the text
// component's content to the file.
    public void actionPerformed(ActionEvent ev)
    { JFileChooser chooser = new JFileChooser(DEFAULT_PATH);
      if (chooser.showSaveDialog(OntologyEditor.this) != JFileChooser.APPROVE_OPTION) return;
      File file = chooser.getSelectedFile();
      if (file == null) return;

      FileOutputStream out;
      PrintStream p;
      try
      { out = new FileOutputStream(file);
        p = new PrintStream(out);
        OwlXml xml = new OwlXml(ontologyText,lexicon);
        String txt = xml.getXmlString();
        p.println(txt);
        p.close();
      }
      catch (IOException ex)
      { JOptionPane.showMessageDialog(OntologyEditor.this,
          "File Not Saved", "ERROR", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

// Listener class for the view menu
  class ViewMenuListener implements ActionListener
  { public void actionPerformed(ActionEvent ev)
    { if (ev.getSource() == viewChecked)
      { String hold = currentTextData;
        String txt = ontologyText.getGeneratedTextString();
        textComp.setText(txt);
        textComp.setCaretPosition(0);
        recolourText();
        previousTextData = hold;
      }
      else if (ev.getSource() == viewGrouped)
      { String hold = currentTextData;
        Glossary glossary = new Glossary(ontologyText,lexicon);
        String txt = glossary.getTextString();
        textComp.setText(txt);
        textComp.setCaretPosition(0);
        recolourText();
        previousTextData = hold;
      }
      else if (ev.getSource() == editUndo)
      { String hold = currentTextData;
        textComp.setText(previousTextData);
        previousTextData = hold;
      }
    }
  }

}
           
         