
/*

Finite-state network for generating English verbalisation of OWL Term

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class OwlEnglish
{ private String textString;
  private OwlTerm owlTerm;
  private int number;
  private boolean definition;

// Syntactic features
  public static final int REJECT = 0;
  public static final int START = 1;
  public static final int NPLIST = 2;
  public static final int VPLIST = 3;
  public static final int VPCHAIN = 4;
  public static final int FINISH = 5;

  public static final int UNSPECIFIED = 0;
  public static final int SINGULAR = 1;
  public static final int PLURAL = 2;

// Test cases
  private static String test[] =
    { "The queen is a head of state.",
      "A white van driver is a driver and a menace to traffic.",
      "The queen is married to the Duke of Edinburgh.",
      "John is aged 24.",
      "John is named \"Mad Dog\".",
      "A van driver drives a vehicle that is made in China.",
      "A van driver drives a vehicle that is made in 1995.",
      "The Duke of Edinburgh has as residence Buckingham Palace and is married to a monarch.",
      "The queen is a monarch and a head of state that is married to the Duke of Edinburgh.",
      "Elizabeth is an octogenerian and a queen that is married to a duke and has as son Prince Charles.",
      "Elizabeth is an octogenerian and a queen that has as son Prince Charles and is married to a duke that lives in a palace that is located in a city that is capital of the United Kingdom.",
      "No man is an island",
      "A romantic novelist is defined as a person that is author of a romantic novel.",
      "A bigamist is a person that is married to at least two women.",
      "A person is a man or a woman."
    };

// Main method for testing
  public static void main(String[] args)
  { Sentence sentence;
    Vector<Verb> verbs = new Vector<Verb>();
    Verb v = new Verb("drive","drives");
    verbs.addElement(v);
    v = new Verb("live","lives");
    verbs.addElement(v);
    for (int i=0; i<test.length; i++)
    { sentence = new Sentence(0,test[i],0,verbs);
      System.out.println(sentence.getOwlFunctionalSyntax());
      OwlEnglish english = new OwlEnglish(sentence.getOwlTerm());
      english.display();
    }
  }

  public OwlEnglish(OwlTerm arg1)
  { owlTerm = arg1;
    transduce();
  }
  
  public String getTextString() {return textString;}

  private void transduce()
  { int currentState = START;
    textString = "";
    number = SINGULAR;
    definition = false;
    OwlTerm register = owlTerm;
    while ((currentState != FINISH) && (currentState != REJECT))
    { // System.out.println(currentState);
// Transcode property assertions to class assertions
      if ((currentState == START) && matchObjectPropertyAssertion(register))
      { currentState = START;
        register = refactorObjectPropertyAssertion(register);
      }
      else if ((currentState == START) && matchDataPropertyAssertion(register))
      { currentState = START;
        register = refactorDataPropertyAssertion(register);
      }
      else if ((currentState == START) && matchNounPredicateStatement(register))
      { currentState = NPLIST;
        if (register.getFunctor() == OwlTerm.EQUIVALENT_CLASSES)
        { textString = basicNounPhrase(register.getArgument(0)) + " is defined as";
          definition = true;
        }
        else if (register.getFunctor() == OwlTerm.DISJOINT_CLASSES)
        textString = disjointNounPhrase(register.getArgument(0)) + " is";
        else  
        textString = basicNounPhrase(register.getArgument(0)) + " is";
        textString = openingCapital(textString);
        register = register.getArgument(1);
      }
      else if ((currentState == START) && matchVerbPredicateStatement(register))
      { currentState = VPLIST;
        textString = basicNounPhrase(register.getArgument(0));
        textString = openingCapital(textString);
        register = register.getArgument(1);
      }
      else if ((currentState == START) && matchDomainStatement(register))
      { currentState = FINISH;
        textString = domainStatement(register);
        textString = openingCapital(textString);
      }
      else if ((currentState == START) && matchRangeStatement(register))
      { currentState = FINISH;
        textString = rangeStatement(register);
        textString = openingCapital(textString);
      }
      else if ((currentState == START) && matchObjectPropertyStatement(register))
      { currentState = FINISH;
        textString = objectPropertyStatement(register);
        textString = openingCapital(textString);
      }
      else if ((currentState == START) && matchDataPropertyStatement(register))
      { currentState = FINISH;
        textString = dataPropertyStatement(register);
        textString = openingCapital(textString);
      }
      else if ((currentState == START) && matchObjectPropertyRelationStatement(register))
      { currentState = FINISH;
        textString = objectPropertyRelationStatement(register);
        textString = openingCapital(textString);
      }
      else if ((currentState == NPLIST) && (register.getFunctor() == OwlTerm.CLASS))
      { currentState = FINISH;
        textString = textString + " " + basicNounPhrase(register) + ".";
      }
      else if ((currentState == NPLIST) && matchClassNounIntersection(register))
      { currentState = NPLIST;
        textString = textString + " " + basicNounPhrase(register.getArgument(0)) + " and";
        register = register.getArgument(1);
        number = SINGULAR;
      }
      else if ((currentState == NPLIST) && matchClassNounUnion(register))
      { currentState = NPLIST;
        textString = textString + " " + basicNounPhrase(register.getArgument(0)) + " or";
        register = register.getArgument(1);
        number = SINGULAR;
      }
      else if ((currentState == NPLIST) && matchClassVerbIntersection(register))
      { currentState = VPLIST;
        textString = textString + " " + basicNounPhrase(register.getArgument(0)) + " that";
        register = register.getArgument(1);
      }
      else if ((currentState == VPLIST) && matchSimpleRestriction(register))
      { currentState = FINISH;
        textString = textString + " " + basicVerbPhrase(register) + ".";
      }
      else if ((currentState == VPLIST) && matchRestrictionIntersection(register))
      { currentState = VPLIST;
        textString = textString + " " + basicVerbPhrase(register.getArgument(0)) + " and";
        register = register.getArgument(1);
        number = SINGULAR;
      }
      else if ((currentState == VPLIST) && matchRestrictionUnion(register))
      { currentState = VPLIST;
        textString = textString + " " + basicVerbPhrase(register.getArgument(0)) + " or";
        register = register.getArgument(1);
        number = SINGULAR;
      }
      else if ((currentState == VPLIST) && matchComplexRestriction(register))
      { currentState = VPCHAIN;
        textString = textString + " " + listVerbPhrase(register) + " that";
        register = register.getArgument(1).getArgument(1);
      }
      else if ((currentState == VPCHAIN) && matchSimpleRestriction(register))
      { currentState = FINISH;
        textString = textString + " " + basicVerbPhrase(register) + ".";
      }
      else if ((currentState == VPCHAIN) && matchComplexRestriction(register))
      { currentState = VPCHAIN;
        textString = textString + " " + listVerbPhrase(register) + " that";
        register = register.getArgument(1).getArgument(1);
      }
      else currentState = REJECT;
    }
  }

// Recognising OwlTerm patterns

// Axiom with individual/class as first argument
  private boolean matchIndividualOrClassStatement(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 2) &&
        ( (term.getFunctor() == OwlTerm.CLASS_ASSERTION) ||
          (term.getFunctor() == OwlTerm.SUB_CLASS_OF) ||
          (term.getFunctor() == OwlTerm.EQUIVALENT_CLASSES) ||
          (term.getFunctor() == OwlTerm.DISJOINT_CLASSES)) &&
        (matchIndividualOrClassEntity(term.getArgument(0)) == true))
    tval = true;
    return tval;
  }

// Axiom with NPLIST as predicate
  private boolean matchNounPredicateStatement(OwlTerm term)
  { boolean tval = false;
    if ((matchIndividualOrClassStatement(term) == true) &&
        (matchNounPredicate(term.getArgument(1)) == true))
    tval = true;
    return tval;
  }

// Axiom with VPLIST as predicate
  private boolean matchVerbPredicateStatement(OwlTerm term)
  { boolean tval = false;
    if ((matchIndividualOrClassStatement(term) == true) &&
        (matchVerbPredicate(term.getArgument(1)) == true))
    tval = true;
    return tval;
  }

// Axiom with OBJECT_PROPERTY_DOMAIN
  private boolean matchDomainStatement(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 2) &&
        ( (term.getFunctor() == OwlTerm.OBJECT_PROPERTY_DOMAIN) ||
          (term.getFunctor() == OwlTerm.DATA_PROPERTY_DOMAIN) ))
    tval = true;
    return tval;
  }

// Axiom with object property attribute
  private boolean matchObjectPropertyStatement(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 1) &&
        ( (term.getFunctor() == OwlTerm.FUNCTIONAL_OBJECT_PROPERTY) ||
          (term.getFunctor() == OwlTerm.INVERSE_FUNCTIONAL_OBJECT_PROPERTY) ||
          (term.getFunctor() == OwlTerm.SYMMETRIC_OBJECT_PROPERTY) ||
          (term.getFunctor() == OwlTerm.ASYMMETRIC_OBJECT_PROPERTY) ||
          (term.getFunctor() == OwlTerm.REFLEXIVE_OBJECT_PROPERTY) ||
          (term.getFunctor() == OwlTerm.IRREFLEXIVE_OBJECT_PROPERTY) ||
          (term.getFunctor() == OwlTerm.TRANSITIVE_OBJECT_PROPERTY) ))
    tval = true;
    return tval;
  }

// Axiom with object property relation
  private boolean matchObjectPropertyRelationStatement(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 2) &&
        ( (term.getFunctor() == OwlTerm.DISJOINT_OBJECT_PROPERTIES) ||
          (term.getFunctor() == OwlTerm.EQUIVALENT_OBJECT_PROPERTIES) ||
          (term.getFunctor() == OwlTerm.INVERSE_OBJECT_PROPERTIES) ||
          (term.getFunctor() == OwlTerm.SUB_OBJECT_PROPERTY_OF) ))
    tval = true;
    return tval;
  }

// Axiom with data property attribute
  private boolean matchDataPropertyStatement(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 1) &&
        (term.getFunctor() == OwlTerm.FUNCTIONAL_DATA_PROPERTY))
    tval = true;
    return tval;
  }

// Axiom with OBJECT_PROPERTY_RANGE
  private boolean matchRangeStatement(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 2) &&
        (term.getFunctor() == OwlTerm.OBJECT_PROPERTY_RANGE))
    tval = true;
    return tval;
  }

// Axiom with OBJECT_PROPERTY_ASSERTION
  private boolean matchObjectPropertyAssertion(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 3) &&
        (term.getFunctor() == OwlTerm.OBJECT_PROPERTY_ASSERTION))
    tval = true;
    return tval;
  }

// Axiom with DATA_PROPERTY_ASSERTION
  private boolean matchDataPropertyAssertion(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 3) &&
        (term.getFunctor() == OwlTerm.DATA_PROPERTY_ASSERTION))
    tval = true;
    return tval;
  }

// Individual or class entity
  private boolean matchIndividualOrClassEntity(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.NAMED_INDIVIDUAL) ||
        (term.getFunctor() == OwlTerm.CLASS))
    tval = true;
    return tval;
  }

// Individual or class entity or literal
  private boolean matchRestrictionValueEntity(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.NAMED_INDIVIDUAL) ||
        (term.getFunctor() == OwlTerm.CLASS) ||
        (term.getFunctor() == OwlTerm.LITERAL))
    tval = true;
    return tval;
  }

// Restriction
  private boolean matchRestriction(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.OBJECT_SOME_VALUES_FROM) ||
        (term.getFunctor() == OwlTerm.OBJECT_HAS_VALUE) ||
        (term.getFunctor() == OwlTerm.DATA_HAS_VALUE) ||
        (term.getFunctor() == OwlTerm.OBJECT_ALL_VALUES_FROM) ||
        (term.getFunctor() == OwlTerm.OBJECT_EXACT_CARDINALITY) ||
        (term.getFunctor() == OwlTerm.OBJECT_MAX_CARDINALITY) ||
        (term.getFunctor() == OwlTerm.OBJECT_MIN_CARDINALITY))
    tval = true;
    return tval;
  }

// Simple restriction, with atomic entity as second argument
  private boolean matchSimpleRestriction(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 2) &&
        (matchRestriction(term) == true) &&
        (matchRestrictionValueEntity(term.getArgument(1)) == true))
    tval = true;
    return tval;
  }

// Simple restriction, with class intersection as second argument
  private boolean matchComplexRestriction(OwlTerm term)
  { boolean tval = false;
    if ((term.getNumberOfArguments() == 2) &&
        (matchRestriction(term) == true) &&
        (matchClassIntersection(term.getArgument(1)) == true))
    tval = true;
    return tval;
  }

// Predicate to be expressed by "is" + NP
  private boolean matchNounPredicate(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.CLASS) ||
        (matchClassIntersection(term) == true) ||
        (matchClassUnion(term) == true))
    tval = true;
    return tval;
  }

// Predicate to be expressed by VP using a property name
  private boolean matchVerbPredicate(OwlTerm term)
  { boolean tval = false;
    if ((matchSimpleRestriction(term) == true) ||
        (matchComplexRestriction(term) == true) ||
        (matchRestrictionIntersection(term) == true) ||
        (matchRestrictionUnion(term) == true))
    tval = true;
    return tval;
  }

// Intersection in which first argument is a class
  private boolean matchClassIntersection(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.OBJECT_INTERSECTION_OF) &&
        (term.getNumberOfArguments() == 2) &&
        (term.getArgument(0).getFunctor() == OwlTerm.CLASS))
    tval = true;
    return tval;
  }

// Intersection in which first argument is a class with NP continuation
  private boolean matchClassNounIntersection(OwlTerm term)
  { boolean tval = false;
    if ((matchClassIntersection(term) == true) &&
        (matchNounPredicate(term.getArgument(1))))
    tval = true;
    return tval;
  }

// Intersection in which first argument is a class with VP continuation
  private boolean matchClassVerbIntersection(OwlTerm term)
  { boolean tval = false;
    if ((matchClassIntersection(term) == true) &&
        (matchVerbPredicate(term.getArgument(1))))
    tval = true;
    return tval;
  }

// Intersection in which first argument is a simple restriction
  private boolean matchRestrictionIntersection(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.OBJECT_INTERSECTION_OF) &&
        (term.getNumberOfArguments() == 2) &&
        (matchSimpleRestriction(term.getArgument(0)) == true))
    tval = true;
    return tval;
  }

// Union in which first argument is a class
  private boolean matchClassUnion(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.OBJECT_UNION_OF) &&
        (term.getNumberOfArguments() == 2) &&
        (term.getArgument(0).getFunctor() == OwlTerm.CLASS))
    tval = true;
    return tval;
  }

// Union in which first argument is a class with NP continuation
  private boolean matchClassNounUnion(OwlTerm term)
  { boolean tval = false;
    if ((matchClassUnion(term) == true) &&
        (matchNounPredicate(term.getArgument(1))))
    tval = true;
    return tval;
  }

// Union in which first argument is a simple restriction
  private boolean matchRestrictionUnion(OwlTerm term)
  { boolean tval = false;
    if ((term.getFunctor() == OwlTerm.OBJECT_UNION_OF) &&
        (term.getNumberOfArguments() == 2) &&
        (matchSimpleRestriction(term.getArgument(0)) == true))
    tval = true;
    return tval;
  }

// Refactoring alternative formulations

  private OwlTerm refactorObjectPropertyAssertion(OwlTerm term)
  { OwlTerm newTerm = new OwlTerm(OwlTerm.CLASS_ASSERTION,2);
    OwlTerm newArg = new OwlTerm(OwlTerm.OBJECT_HAS_VALUE,2);
    newTerm.setArgument(0,term.getArgument(1));
    newArg.setArgument(0,term.getArgument(0));
    newArg.setArgument(1,term.getArgument(2));
    newTerm.setArgument(1,newArg);
    return newTerm;
  }
    
  private OwlTerm refactorDataPropertyAssertion(OwlTerm term)
  { OwlTerm newTerm = new OwlTerm(OwlTerm.CLASS_ASSERTION,2);
    OwlTerm newArg = new OwlTerm(OwlTerm.DATA_HAS_VALUE,2);
    newTerm.setArgument(0,term.getArgument(1));
    newArg.setArgument(0,term.getArgument(0));
    newArg.setArgument(1,term.getArgument(2));
    newTerm.setArgument(1,newArg);
    return newTerm;
  }

// Generating phrase strings

// Noun group without determiner
  private String basicNounGroup(OwlTerm term)
  { String s = englishName(term);
    return s;
  }

// Named individual or class as subject or complement
  private String basicNounPhrase(OwlTerm term)
  { String s = englishName(term);
//    if (definition == true) s = "any " + s;
    if (term.getFunctor() == OwlTerm.CLASS) s = indefiniteArticle(s) + " " + s;
    definition = false;
    return s;
  }

// Class as subject in a disjoint classes statement
  private String disjointNounPhrase(OwlTerm term)
  { String s = englishName(term);
    if (term.getFunctor() == OwlTerm.CLASS) s = "No " + s;
    return s;
  }

// Restriction with simple individual/class value
  private String basicVerbPhrase(OwlTerm term)
  { String verb = englishName(term.getArgument(0));
    String det = determiner(term,term.getArgument(1));
    String np = basicNounGroup(term.getArgument(1));
    String s;
    if (det.equals("")) s = verb + " " + np;
    else s = verb + " " + det + " " + np;
    return s;
  }

// Restriction with class intersection as value
  private String listVerbPhrase(OwlTerm term)
  { String verb = englishName(term.getArgument(0));
    String det = determiner(term,term.getArgument(1).getArgument(0));
    String np = basicNounGroup(term.getArgument(1).getArgument(0));
    String s;
    if (det.equals("")) s = verb + " " + np;
    else s = verb + " " + det + " " + np;
    return s;
  }

// Determiner in a restriction (possibly plural)
  private String determiner(OwlTerm term, OwlTerm arg)
  { String s = "";
    if (term.getFunctor() == OwlTerm.OBJECT_SOME_VALUES_FROM)
    { number = SINGULAR;
      s = indefiniteArticle(arg.getEntityName());
    }
    else if (term.getFunctor() == OwlTerm.OBJECT_HAS_VALUE)
    { number = SINGULAR;
      s = "";
    }
    else if (term.getFunctor() == OwlTerm.DATA_HAS_VALUE)
    { number = SINGULAR;
      s = "";
    }
    else if (term.getFunctor() == OwlTerm.OBJECT_ALL_VALUES_FROM)
    { number = PLURAL;
      s = "only";
    }
    else if (term.getFunctor() == OwlTerm.OBJECT_EXACT_CARDINALITY)
    { s = term.getEntityName();
      if (singularQuantifier(s) == true) number = SINGULAR;
      else number = PLURAL;
      s = "exactly " + s;
    }
    else if (term.getFunctor() == OwlTerm.OBJECT_MAX_CARDINALITY)
    { s = term.getEntityName();
      if (singularQuantifier(s) == true) number = SINGULAR;
      else number = PLURAL;
      s = "at most " + s;
    }
    else if (term.getFunctor() == OwlTerm.OBJECT_MIN_CARDINALITY)
    { s = term.getEntityName();
      if (singularQuantifier(s) == true) number = SINGULAR;
      else number = PLURAL;
      s = "at least " + s;
    }
    return s;
  }

 private boolean singularQuantifier(String s)
  { boolean tval = false;
    if (s.equals("one") || s.equals("1")) tval = true;
    return tval;
  }

// Property domain statement
  private String domainStatement(OwlTerm term)
  { String verb = englishName(term.getArgument(0));
    String np = basicNounPhrase(term.getArgument(1));
    String s = "Anything that " + verb + " something is " + np + ".";
    if (term.getFunctor() == OwlTerm.DATA_PROPERTY_DOMAIN)
    s = "Anything that " + verb + " some value is " + np + ".";
    return s;
  }

// Property range statement
  private String rangeStatement(OwlTerm term)
  { String verb = englishName(term.getArgument(0));
    String np = basicNounPhrase(term.getArgument(1));
    String s = "Anything that something " + verb + " is " + np + ".";
    return s;
  }

// Object property statement
  private String objectPropertyStatement(OwlTerm term)
  { String verb = englishName(term.getArgument(0));
    String noun = attributeName(term.getFunctor());
    String s = "The property <" + verb + "> is " + noun + ".";
    return s;
  }

// Data property statement
  private String dataPropertyStatement(OwlTerm term)
  { String verb = englishName(term.getArgument(0));
    String noun = attributeName(term.getFunctor());
    String s = "The data-property <" + verb + "> is " + noun + ".";
    return s;
  }

// Object property relation statement
  private String objectPropertyRelationStatement(OwlTerm term)
  { String s = "";
    if (term.getFunctor() == OwlTerm.SUB_OBJECT_PROPERTY_OF)
    { s = subObjectPropertyStatement(term); }
    else
    { s = mainObjectPropertyRelationStatement(term); }
    return s;
  }

// Object sub-property statement
  private String subObjectPropertyStatement(OwlTerm term)
  { String verb1 = englishName(term.getArgument(0));
    String verb2 = englishName(term.getArgument(1));
    String s = "The property <" + verb1 + "> is a sub-property of <" + verb2 + ">.";
    return s;
  }

// Other object-property relation statement
  private String mainObjectPropertyRelationStatement(OwlTerm term)
  { String verb1 = englishName(term.getArgument(0));
    String verb2 = englishName(term.getArgument(1));
    String relation = relationName(term.getFunctor());
    String s = "The properties <" + verb1 + "> and <" + verb2 + "> are " + relation + ".";
    return s;
  }

// English name for atomic term
  private String englishName(OwlTerm term)
  { String name;
    if (number == SINGULAR) name = term.getEntityName();
    else name = term.getPluralEntityName();
    return name;
  }

// Names for property attributes
  private String attributeName(int functor)
  { String name = "unspecified";
    if (functor == OwlTerm.FUNCTIONAL_OBJECT_PROPERTY) name = "functional";
    else if (functor == OwlTerm.FUNCTIONAL_DATA_PROPERTY) name = "functional";
    else if (functor == OwlTerm.INVERSE_FUNCTIONAL_OBJECT_PROPERTY) name = "inverse-functional";
    else if (functor == OwlTerm.SYMMETRIC_OBJECT_PROPERTY) name = "symmetric";
    else if (functor == OwlTerm.ASYMMETRIC_OBJECT_PROPERTY) name = "asymmetric";
    else if (functor == OwlTerm.REFLEXIVE_OBJECT_PROPERTY) name = "reflexive";
    else if (functor == OwlTerm.IRREFLEXIVE_OBJECT_PROPERTY) name = "irreflexive";
    else if (functor == OwlTerm.TRANSITIVE_OBJECT_PROPERTY) name = "transitive";
    return name;
  }

// Names for property attributes
  private String relationName(int functor)
  { String name = "unspecified";
    if (functor == OwlTerm.DISJOINT_OBJECT_PROPERTIES) name = "disjoint";
    else if (functor == OwlTerm.EQUIVALENT_OBJECT_PROPERTIES) name = "equivalent";
    else if (functor == OwlTerm.INVERSE_OBJECT_PROPERTIES) name = "inverse";
    return name;
  }

// Make sure the opening letter is upper-case at start of sentence
  private String openingCapital(String s)
  { String cs = s.substring(0,1).toUpperCase()+ s.substring(1,s.length());
    return cs;
  }

  private String indefiniteArticle(String noun)
  { String indef = "a";
    if ((noun.charAt(0)=='a') ||
        (noun.charAt(0)=='e') ||
        (noun.charAt(0)=='i') ||
        (noun.charAt(0)=='o') ||
        (noun.charAt(0)=='u') ||
        (noun.charAt(0)=='A') ||
        (noun.charAt(0)=='E') ||
        (noun.charAt(0)=='I') ||
        (noun.charAt(0)=='O') ||
        (noun.charAt(0)=='U'))
    indef = "an";
    return indef;
  }
 
  public void display()
  { System.out.println(textString);
  }

}
