import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.util.Vector;
import java.util.Collections;
import java.util.*;

import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;

/*

Imports OWL/XML to a text in Owl English given a document as input

Modifier 01-06-2012 to replace identifiers with labels, if they exist.
Checks only for annotation assertions with property rdfs:label.

*/

public class OwlImport
{ private Document document;
  private OwlTerm ontology;
  private Vector<String> verbs;
  private String ontologyIRI;
  private String textString;
  private Vector<String> annotations;
  private Vector<String> labels;

  public OwlImport(File file)
  { try
    { DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      document = builder.parse(file);
      ontologyIRI = "Anonymous";
      verbs = new Vector<String>();
      Element root = document.getDocumentElement();
      makeLabels(root);
      ontology = makeOwlTerm(root,0);
      textString = makeTextString();
    }
    catch (SAXException ex)
    { textString = "Error in file";
    }
    catch (IOException ex)
    { textString = "Error in file";
    }
    catch (ParserConfigurationException ex)
    { textString = "Error in file";
    }
  }

  public String getTextString() { return textString; }



  private void makeLabels(Node node)
  { annotations = new Vector<String>();
    labels = new Vector<String>();
    NodeList list = node.getChildNodes();
    Node child;
    String name;
    for (int i = 0; i < list.getLength(); i++)
    { child = list.item(i);
      name = child.getNodeName();
      if (name.equals("AnnotationAssertion")) makeAnnotation(child);
    }
  }

  private void makeAnnotation(Node node)
  { NodeList list = node.getChildNodes();
    Node child;
    String name;
    String prop = "";
    String iri = "";
    String lit = "";
    Identifier id;
    for (int i = 0; i < list.getLength(); i++)
    { child = list.item(i);
      name = child.getNodeName();
      if (name.equals("AnnotationProperty"))
      { prop = ((Element)child).getAttribute("abbreviatedIRI");
        id = new Identifier(OwlTerm.CLASS,prop);
        prop = id.getEntityName();
      }
      else if (name.equals("AbbreviatedIRI") || name.equals("IRI"))
      { iri = ((Element)child).getTextContent();
        id = new Identifier(OwlTerm.NAMED_INDIVIDUAL,iri);
        iri = id.getEntityName();
      }
      else if (name.equals("Literal"))
      { lit = ((Element)child).getTextContent();
        id = new Identifier(OwlTerm.LITERAL,lit);
        lit = id.getEntityName();
      }
    }
    if (prop.equals("label"))
    { annotations.addElement(iri);
      labels.addElement(lit);
    }
  }

  private OwlTerm makeOwlTerm(Node node, int functor)
  { Vector<OwlTerm> args = getArguments(node);
    int nArgs = args.size();
    OwlTerm term = new OwlTerm(functor,nArgs);
    String attr;
    Identifier identifier;
    for (int i=0; i<nArgs; i++) term.setArgument(i,args.elementAt(i));
    if ((functor == OwlTerm.NAMED_INDIVIDUAL) ||
        (functor == OwlTerm.CLASS) ||
        (functor == OwlTerm.OBJECT_PROPERTY) ||
        (functor == OwlTerm.DATA_PROPERTY))
    { attr = ((Element)node).getAttribute("IRI");
      if (attr.equals("")) attr = ((Element)node).getAttribute("abbreviatedIRI");
      
      identifier = new Identifier(functor,attr);
      String name = identifier.getEntityName();

      for (int i=0; i<annotations.size(); i++)
      { if (annotations.elementAt(i).equalsIgnoreCase(name))
        identifier = new Identifier(functor,labels.elementAt(i));
      }

      term.setEntityName(identifier.getEnglishName());
      updateVerbs(functor,identifier.getEnglishName());
    }
    else if (functor == OwlTerm.LITERAL)
    term.setEntityName(((Element)node).getTextContent());

    else if ((functor == OwlTerm.OBJECT_MIN_CARDINALITY) ||
             (functor == OwlTerm.OBJECT_MAX_CARDINALITY) ||
             (functor == OwlTerm.OBJECT_EXACT_CARDINALITY) ||
             (functor == OwlTerm.DATA_MIN_CARDINALITY) ||
             (functor == OwlTerm.DATA_MAX_CARDINALITY) ||
             (functor == OwlTerm.DATA_EXACT_CARDINALITY))
    term.setEntityName(((Element)node).getAttribute("cardinality"));

    else if (functor == 0)
    { attr = ((Element)node).getAttribute("xml:base");
      if (attr.length() > 0) ontologyIRI = attr;
    }

//  Switch argument order for ClassAssertion
    if (functor == OwlTerm.CLASS_ASSERTION)
    { OwlTerm arg0 = term.getArgument(0);
      term.setArgument(0,term.getArgument(1));
      term.setArgument(1,arg0);
    }

    return term;
  }

  private Vector<OwlTerm> getArguments(Node node)
  { Vector<OwlTerm> args = new Vector<OwlTerm>();
    NodeList list = node.getChildNodes();
    OwlTerm newTerm;
    for (int i = 0; i < list.getLength(); i++)
    { Node child = list.item(i);
      String name = child.getNodeName();
      int functor = getOwlFunctor(name);
      if (functor > 1)
      { newTerm = makeOwlTerm(child,functor);
        args.addElement(newTerm);
      }
    }
    return args;
  }

  private int getOwlFunctor(String name)
  { int functor = 0;
    for (int i=0; i<OwlTerm.functorString.length; i++)
    if (name.equals(OwlTerm.functorString[i])) functor = i;
    return functor;
  }

// Add verb to list if not already there
  private void updateVerbs(int functor, String name)
  { if ((functor == OwlTerm.OBJECT_PROPERTY) ||
        (functor == OwlTerm.DATA_PROPERTY))
    { String verb = firstWord(name);
      if ((!verb.equals("is")) &&
          (!verb.equals("has")) &&
          (!verb.equals("")) &&
          (!verbs.contains(verb)))
      verbs.addElement(verb);
    }
  }

// Pick first word from a string
  private String firstWord(String name)
  { String s = "";
    String ss[] = name.split(" ");
    if (ss.length > 0) s = ss[0];
    return s;
  }

  private String makeTextString()
  { String s = "# OntologyIRI " + ontologyIRI + "\n";
    String singular;
    String plural;
    for (int i=0; i<verbs.size(); i++)
    { singular = verbs.elementAt(i);
      if (singular.charAt(singular.length()-1)=='s')
      plural = singular.substring(0,singular.length()-1);
      else
      { plural = singular;
        singular = plural + "s";
      }
      s = s + "# Verb " + plural + " " + singular + "\n";
    }
    int nAxioms = ontology.getNumberOfArguments();
    int nSentences = 0;
    for (int i=0; i<nAxioms; i++)
    { OwlEnglish english = new OwlEnglish(ontology.getArgument(i));
      if (english.getTextString().length() > 0)
      { s = s + english.getTextString() + "\n";
        nSentences++;
      }
      else 
      { String prform = ontology.getArgument(i).getPrintForm();
        System.out.println(prform);
      }
    }
    System.out.println("Axioms accepted = " + nSentences + "/" + nAxioms);
    return s;
  }

}