
/*

Compute singular and plural forms of nouns (-s suffix)

Split the singular form into an opening and ending, where the ending
is the last two characters.

  day = d + ay
  fairy = fai + ry
  life = li + fe
  dish = di + sh
  princess = prince + ss
  fox = f + ox

Then apply these transformations to the ending

  Vy => Vys
  Cy => Cies
  fe => ves
  sh => shes
  ch => ches
  #s => #ses
  #x => #xes

Or ## => ##s if none of these matches.

Here V matches vowel, C consonant, # any letter

For plural-to-singular the rule depends on the final four characters:

 #Vys => #Vy
 Cies => Cy
 #ves => #fe
 shes => sh
 ches => ch
 #ses => #s
 #xes => #x

Or ###s => ###  or #### => #### if none of these matches.

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class WordForm
{ private String singular;
  private String plural;

  public static final int SINGULAR = 0;
  public static final int PLURAL = 1;

// Irregular plurals
  private static String irregular[][] =
    { { "man", "men" },
      { "woman", "women" },
      { "child", "children" },
      { "person", "people" },
      { "mouse", "mice" }
    }; 

// Test cases
  private static String test0[] =
    { "day",
      "fairy",
      "life",
      "dish",
      "princess",
      "fox",
      "person",
      "cup"
    };

// Test cases
  private static String test1[] =
    { "days",
      "fairies",
      "lives",
      "dishes",
      "princesses",
      "foxes",
      "people",
      "cups",
      "glass",
      "paper"
    };

// Main method for testing
  public static void main(String[] args)
  { WordForm wordForm;
    for (int i=0; i<test0.length; i++)
    { wordForm = new WordForm(test0[i],SINGULAR);
      wordForm.display();
    }
    System.out.println("-------------");
    for (int i=0; i<test1.length; i++)
    { wordForm = new WordForm(test1[i],PLURAL);
      wordForm.display();
    }
  }

  public WordForm(String arg1, int arg2)
  { if (arg2 == SINGULAR)
    { singular = arg1;
      plural = irregularPlural(singular);
      if (plural.length()==0) plural = makePlural(singular);
    }
    else if (arg2 == PLURAL)
    { plural = arg1;
      singular = irregularSingular(plural);
      if (singular.length()==0) singular = makeSingular(plural);
    }
  }

  public String getSingular() {return singular; }
  public String getPlural() {return plural; }

  private String irregularPlural(String s)
  { String p = "";
    for (int i=0; i<irregular.length; i++)
    if (s.equals(irregular[i][0])) p = irregular[i][1];
    return p;
  }

  private String irregularSingular(String p)
  { String s = "";
    for (int i=0; i<irregular.length; i++)
    if (p.equals(irregular[i][1])) s = irregular[i][0];
    return s;
  }

  private String makePlural(String s)
  { String p;
    int len = s.length();
    if (len > 2)
    { String opening = s.substring(0,len-2);
      String ending = s.substring(len-2,len);
      p = opening + makePluralEnding(ending);
    }
    else p = s + "s";
    return p; 
  }

// Transformation on final two characters of singular form
  private String makePluralEnding(String s)
  { String p;
    char c0 = s.charAt(0);
    char c1 = s.charAt(1);
    if (!vowel(c0)&&(c1=='y')) p = c0 + "ies";
    else if ((c0=='f')&&(c1=='e')) p = "ves";
    else if ((c0=='s')&&(c1=='h')) p = "shes";
    else if ((c0=='c')&&(c1=='h')) p = "ches";
    else if (c1=='s') p = c0 + "ses";
    else if (c1=='x') p = c0 + "xes";
    else p = s + "s";
    return p; 
  }

  private String makeSingular(String p)
  { String s;
    int len = p.length();
    if (len > 3)
    { String opening = p.substring(0,len-4);
      String ending = p.substring(len-4,len);
      s = opening + makeSingularEnding(ending);
    }
    else s = p;
    return s; 
  }

// Transformation on final four characters of plural form
  private String makeSingularEnding(String p)
  { String s;
    char c0 = p.charAt(0);
    char c1 = p.charAt(1);
    char c2 = p.charAt(2);
    char c3 = p.charAt(3);
    if (!vowel(c0)&&(c1=='i')&&(c2=='e')&&(c3=='s')) s = c0 + "y";
    else if ((c1=='v')&&(c2=='e')&&(c3=='s')) s = c0 + "fe";
    else if ((c0=='s')&&(c1=='h')&&(c2=='e')&&(c3=='s')) s = "sh";
    else if ((c0=='c')&&(c1=='h')&&(c2=='e')&&(c3=='s')) s = "ch";
    else if ((c1=='s')&&(c2=='e')&&(c3=='s')) s = c0 + "s";
    else if ((c1=='x')&&(c2=='e')&&(c3=='s')) s = c0 + "x";
    else if ((c2!='s')&&(c3=='s')) s = p.substring(0,3);
    else s = p;
    return s; 
  }

  private boolean vowel(char c)
  { boolean tval = false;
    if ((c=='a')||(c=='e')||(c=='o')||(c=='i')||(c=='u'))
    tval = true;
    return tval;
  }

  public void display()
  { System.out.println(singular + " " + plural);
  }

}
